/*
 * This Java source file is completely Free and Open-Source, 
 * released under the MIT License.
 *
 * Copyright (2012) Ashish Nawal Saharia (ashish.saharia@yahoo.co.in)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to deal 
 * in the Software without restriction, including without limitation the rights 
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
 * OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 */

/*
 * DBConnParam.java
 *
 * Created on 27 December 2008, 05:44
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package net.infantai.dbcp.secure;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 *
 * @author Ashish (ashish.saharia@yahoo.co.in)
 */
public class DBConnParam
{
    
//    private static DBConnParam dbParam = null;
    
    private String dbDriverClassName = null;
    private String dbServer = null;
    private String dbServerPort = null;
    private String dbName = null;
    private String dbUsername = null;
    private String dbPassword = null;
    private String dbConnectionString = null;
    
    private String kDString = "Ashish Nawal Saharia, 2018."; // Key-derivation string. Yes, hard-coded, sorry about that. Can get it from a custom key-file later.
    
    // Database Connection Pool Parameters
    private int maxPoolSize = 150;
    private int maxWait = 15000; // Milliseconds
    
    private String poolIdentifier = null;
    private static Map<String, DBConnParam> poolParam = new LinkedHashMap<String, DBConnParam>();
    
    private DBConnParam(String poolIdentifier)
    {
        this.poolIdentifier = poolIdentifier;
    }
    
    public static DBConnParam getInstance(String poolIdentifier)
    {
        if (poolParam.get(poolIdentifier) == null)
        {
            DBConnParam dbParam = new DBConnParam(poolIdentifier);
            poolParam.put(poolIdentifier, dbParam);
        }
        return poolParam.get(poolIdentifier);
    }
    
    public void setParam(String dbDriverClassName, String dbServer, String dbServerPort, String dbName, String dbUsername, String dbPassword, String dbConnectionString)
    {
        this.dbDriverClassName = dbDriverClassName;
        this.dbServer = dbServer;
        this.dbServerPort = dbServerPort;
        this.dbName = dbName;
        this.dbUsername = dbUsername;
        this.dbPassword = dbPassword;
        this.dbConnectionString = dbConnectionString;    
    }

    public String getDBDriverClassName() 
    {
        return dbDriverClassName;
    }

    public void setDBDriverClassName(String dbDriverClassName) 
    {
        this.dbDriverClassName = dbDriverClassName;
    }

    public String getDBServer() 
    {
        return dbServer;
    }

    public void setDBServer(String dbServer) 
    {
        this.dbServer = dbServer;
    }

    public String getDBServerPort() 
    {
        return dbServerPort;
    }

    public void setDBServerPort(String dbServerPort) 
    {
        this.dbServerPort = dbServerPort;
    }

    public String getDBName() 
    {
        return dbName;
    }

    public void setDBName(String dbName) 
    {
        this.dbName = dbName;
    }

    public String getDBUsername() 
    {
        return dbUsername;
    }

    public void setDBUsername(String dbUsername) 
    {
        this.dbUsername = dbUsername;
    }

    public String getDBPassword() 
    {
        return dbPassword;
    }

    public void setDBPassword(String dbPassword) 
    {
        this.dbPassword = dbPassword;
    }

    public String getDBConnectionString() 
    {
        return dbConnectionString;
    }

    public void setDBConnectionString(String dbConnectionString) 
    {
        this.dbConnectionString = dbConnectionString;
    }
    
    public int getMaxPoolSize() 
    {
        return maxPoolSize;
    }

    public void setMaxPoolSize(int maxPoolSize) 
    {
        this.maxPoolSize = maxPoolSize;
    }

    public int getMaxWait() 
    {
        return maxWait;
    }

    public void setMaxWait(int maxWait) 
    {
        this.maxWait = maxWait;
    }
    
    /**
     * @return the poolIdentifier
     */
    public String getPoolIdentifier()
    {
        return this.poolIdentifier;
    }
    
    /**
     * @return the kDString
     */
    public String getKDString()
    {
        return kDString;
    }

    /**
     * @param kDString the kDString to set
     */
    public void setKDString(String kDString)
    {
        this.kDString = kDString;
    }    
    
}
