/*
 * This Java source file is completely Free and Open-Source, 
 * released under the MIT License.
 *
 * Copyright (2012) Ashish Nawal Saharia (ashish.saharia@yahoo.co.in)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to deal 
 * in the Software without restriction, including without limitation the rights 
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
 * OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 */

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package net.infantai.nlp;

import java.text.Normalizer;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import net.infantai.nlp.ner.EntityTaggerOutput;
import net.infantai.util.common.TextUtil;
import net.infantai.util.common.TokenGroup;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 *
 * @author Ashish (ashish.saharia@yahoo.co.in)
 */
public class RegExTagger
{
    private static final Logger logger = LogManager.getLogger(RegExTagger.class);
    
    private static final Object objLock = new Object();
    
    private static final TextUtil tu = new TextUtil();
    
    private static final double MAX_CONFIDENCE_SCORE = 0.85;
    
    private static final String REGEX_FOR_WHITESPACE = "[ \r\n	    ]+";
    
    public RegExTagger()
    {
        // Default constructor, nothing to do here.
    }
    
    /**
     * Returns a RegEx Pattern in which group index 2 will contain the required match.
     * @param elementsToMatch
     * @return 
     */
    public Pattern buildRegExPatternForQuantities(List<String> elementsToMatch, boolean escapeDotCharacter, boolean spaceRepresentsNonWordCharacters)
    {
        Pattern pat = null;
        try
        {
            StringBuilder sbRegexForCapture = new StringBuilder();
            String s1 = "(\\A|\\W+)(";
            sbRegexForCapture.append(s1);
            for (int i = 0; i < elementsToMatch.size(); ++i)
            {
                String regexPart = elementsToMatch.get(i).replace("?", "[\\d,.]+?\\s*");
                if (escapeDotCharacter)
                {
                    regexPart = regexPart.replace(".", "\\.");
                }
                if (spaceRepresentsNonWordCharacters)
                {
//                    regexPart = regexPart.replaceAll("\\s+", REGEX_FOR_WHITESPACE);
                    regexPart = regexPart.replaceAll("\\s+", "\\\\W+");
                }
                sbRegexForCapture.append(regexPart);
                if (i < (elementsToMatch.size() - 1))
                {
                    sbRegexForCapture.append('|');
                }
            }
            String s2 = ")(\\W+|\\Z)";
            sbRegexForCapture.append(s2);
            String strPat = sbRegexForCapture.toString();
//            System.out.println("=> " + strPat.replaceAll("\r\n", ""));
            pat = Pattern.compile(strPat, Pattern.CASE_INSENSITIVE | Pattern.MULTILINE | Pattern.DOTALL);
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
        return pat;
    }
    
    public String normalizeNonASCIIText(String input)
    {
        return Normalizer.normalize(input, Normalizer.Form.NFD);
    }
    
    public List<EntityTaggerOutput> getQuantities(String input, Pattern pat)
    {
        List<EntityTaggerOutput> matches = new ArrayList<EntityTaggerOutput>();
        Matcher mat = pat.matcher(input);
        while (mat.find())
        {
//            System.out.println("Found " + mat.group(1));
            EntityTaggerOutput eto = new EntityTaggerOutput();
            String entityName = mat.group(2).trim();
            int startIndex = mat.start(2);
            TokenGroup tg = new TokenGroup(entityName, startIndex, entityName.length());
            eto.tg = tg;
            eto.isDictionaryMatch = false;
            eto.entityType = net.infantai.nlp.ner.Constants.ENTITY_TYPE_ORGANIZATION;
            eto.confidenceScore = MAX_CONFIDENCE_SCORE;
            matches.add(eto);
        }        
        return matches;
    }    
    
    public List<EntityTaggerOutput> getQuantities(String input, Pattern pat, List<String> textToDiscardFromBeginning, List<String> textToDiscardFromEnd)
    {
        List<EntityTaggerOutput> matches = new ArrayList<EntityTaggerOutput>();
        Matcher mat = pat.matcher(input);
        while (mat.find())
        {
//            System.out.println("Found " + mat.group(1));
            EntityTaggerOutput eto = new EntityTaggerOutput();
            String entityName = mat.group(2).trim();
            int startIndex = mat.start(2);
            
            for (int i = 0; i < textToDiscardFromBeginning.size(); ++i)
            {
                String s1 = textToDiscardFromBeginning.get(i); // expect a lower case string here, no need to call toLowerCase here
                int i1 = 0;
                int i2 = s1.length();
                if ((i2 - i1) < (entityName.length() - 1))
                {
                    String s2 = entityName.substring(i1, i2).toLowerCase();
                    if (s1.equals(s2))
                    {
                        entityName = entityName.substring((i2 + 1));
                        startIndex = mat.start(2) + i2 + 1;
                    }
                }
            }
            
            for (int i = 0; i < textToDiscardFromEnd.size(); ++i)
            {
                String s1 = textToDiscardFromEnd.get(i); // expect a lower case string here, no need to call toLowerCase here
                int i1 = entityName.length() - s1.length();
                int i2 = i1 + s1.length();
                if (i1 >= 0 && i2 <= entityName.length())
                {
                    String s2 = entityName.substring(i1, i2).toLowerCase();
                    if (s1.equals(s2))
                    {
//                        entityName = entityName.substring(0, i1);
                        entityName = entityName.substring(0, i1).trim();
                    }
                }
            }
            
            TokenGroup tg = new TokenGroup(entityName, startIndex, entityName.length());
            eto.tg = tg;
            eto.isDictionaryMatch = false;
            eto.entityType = net.infantai.nlp.ner.Constants.ENTITY_TYPE_QUANTITY;
            eto.confidenceScore = MAX_CONFIDENCE_SCORE;
            matches.add(eto);
        }        
        return matches;
    }    
    
    public List<EntityTaggerOutput> getQuantities(String input, Pattern pat, List<String> textToDiscardFromBeginning, List<String> textToDiscardFromEnd, Set<String> vicinityTokens, int vicinityWC)
    {
        List<EntityTaggerOutput> matches = new ArrayList<EntityTaggerOutput>();
        Matcher mat = pat.matcher(input);
        while (mat.find())
        {
//            System.out.println("Found " + mat.group(1));
            EntityTaggerOutput eto = new EntityTaggerOutput();
            String entityName = mat.group(2).trim();
            int startIndex = mat.start(2);
            
            boolean foundVicinityToken = false;
            TextUtil tu = new TextUtil();
            Set<String> prevTokens = tu.getUpToNTokensFromPrevTextWithSingleTokens(input.substring(0, startIndex), vicinityWC);
            for (String s : prevTokens)
            {
                if (vicinityTokens.contains(s))
                {
                    foundVicinityToken = true;
                    break;
                }
            }
            
            if (!foundVicinityToken)
            {
                Set<String> nextTokens = tu.getUpToNTokensFromNextTextWithSingleTokens(input.substring(startIndex + entityName.length()), vicinityWC);
                for (String s : nextTokens)
                {
                    if (vicinityTokens.contains(s))
                    {
                        foundVicinityToken = true;
                        break;
                    }
                }            
            }
            
            if (foundVicinityToken)
            {
                for (int i = 0; i < textToDiscardFromBeginning.size(); ++i)
                {
                    String s1 = textToDiscardFromBeginning.get(i); // expect a lower case string here, no need to call toLowerCase here
                    int i1 = 0;
                    int i2 = s1.length();
                    if ((i2 - i1) < (entityName.length() - 1))
                    {
                        String s2 = entityName.substring(i1, i2).toLowerCase();
                        if (s1.equals(s2))
                        {
                            entityName = entityName.substring((i2 + 1));
                            startIndex = mat.start(2) + i2 + 1;
                        }
                    }
                }

                for (int i = 0; i < textToDiscardFromEnd.size(); ++i)
                {
                    String s1 = textToDiscardFromEnd.get(i); // expect a lower case string here, no need to call toLowerCase here
                    int i1 = entityName.length() - s1.length();
                    int i2 = i1 + s1.length();
                    if (i1 >= 0 && i2 <= entityName.length())
                    {
                        String s2 = entityName.substring(i1, i2).toLowerCase();
                        if (s1.equals(s2))
                        {
    //                        entityName = entityName.substring(0, i1);
                            entityName = entityName.substring(0, i1).trim();
                        }
                    }
                }

                TokenGroup tg = new TokenGroup(entityName, startIndex, entityName.length());
                eto.tg = tg;
                eto.isDictionaryMatch = false;
                eto.entityType = net.infantai.nlp.ner.Constants.ENTITY_TYPE_QUANTITY;
                eto.confidenceScore = MAX_CONFIDENCE_SCORE;
                matches.add(eto);
            }

        }        
        return matches;
    }        
    
    public static void main(String[] args)
    {
        RegExTagger regexTagger = new RegExTagger();
//        String input = "Tesla, Inc. (formerly Tesla Motors, Inc.) is an American electric vehicle and clean energy company based in Palo Alto, California. Tesla's current products include electric cars (the Model S, Model 3, Model X, and Model Y), battery energy storage from home to grid scale (the Powerwall, Powerpack, and Megapack), solar products (solar panels and solar roof tiles) and related products and services. Founded in July 2003 by engineers Martin Eberhard and Marc Tarpenning as Tesla Motors, the company’s name is a tribute to inventor and electrical engineer Nikola Tesla. The next three employees were Ian Wright, Elon Musk, and J. B. Straubel, all of whom are named as co-founders of the company. Elon Musk, who formerly served as chairman and is the current CEO, said in 2006 that \"the overarching purpose of Tesla Motors...is to help expedite the move from a mine-and-burn hydrocarbon economy towards a solar electric economy\" and it would build a wide range of electric vehicles, including \"affordably priced family cars\", and co-market SolarCity solar panels to do so. Tesla later acquired SolarCity in 2016.";
        String input = "650 mcg ARNA 50 mg Genomics US Inc, which mg develops tests for 40.5MG/ML early stage cancer by detecting 10,000UNITS/ML specific fragments of nucleic acids in plasma, announced today closing of the $3,5 million series A with a leading investor — Xploration Capital venture fund. Raised funding will be used for clinical validation of ARNA Breast — a first unique test for breast cancer.";
        try
        {
            List<String> dosages = new ArrayList<>();
            dosages.add("?mg/?ml");            
            dosages.add("?mg/ml");            
            dosages.add("?mg");
            dosages.add("?milligram");
            dosages.add("?milligrams");
            dosages.add("?mcg");
            dosages.add("?microgram");
            dosages.add("?micrograms");
            dosages.add("?ml");
            dosages.add("?millilitre");
            dosages.add("?millilitres");
            dosages.add("?units/?ml");
            dosages.add("?units/ml");
            dosages.add("?units");
            Pattern pat = regexTagger.buildRegExPatternForQuantities(dosages, true, true);
            regexTagger.getQuantities(input, pat);
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
    }
    
//    public static Set<TokenGroup> getAllDistinctTokens(String input, List<String> delimiters)
//    {
//        Set<TokenGroup> distinctTokens = new TreeSet<>();
//        Tokenizer tokenizer = new Tokenizer();
//        for (int j = 0; j < delimiters.size(); ++j)
//        {
//            distinctTokens.addAll(tokenizer.getAllTokens(input, delimiters.get(j), 6));
//        }
//        return distinctTokens;
//    }    
    
}
