/*
 * This Java source file is completely Free and Open-Source, 
 * released under the MIT License.
 *
 * Copyright (2012) Ashish Nawal Saharia (ashish.saharia@yahoo.co.in)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to deal 
 * in the Software without restriction, including without limitation the rights 
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
 * OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 */

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package net.infantai.nlp.ner;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;
import net.infantai.util.common.TokenGroup;
import org.apache.logging.log4j.Level;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 *
 * @author Ashish (ashish.saharia@yahoo.co.in)
 */
public class EntityProximityRuleValidator
{
    
    private static final Logger LOGGER = LogManager.getLogger(EntityProximityRuleValidator.class);
    
    public static void validateRules(String input, List<? extends EntityTaggerOutput> allTaggedEntities, Map<Long, Set<Long>> preAllowedEntityTypes, Map<Long, Set<Long>> postAllowedEntityTypes, Set<String> allowedIntermediateText)
    {
        Collections.sort(allTaggedEntities);
        
        if (allTaggedEntities.size() == 1)
        {
            EntityTaggerOutput eto = allTaggedEntities.get(0);
            if (preAllowedEntityTypes.containsKey(eto.ek.getEntityTypeId()) || postAllowedEntityTypes.containsKey(eto.ek.getEntityTypeId()))
            {
                LOGGER.debug("Validation rule fails for single Entity, discarding Entity: " + eto.tg.getToken() + ", Id: " + eto.ek.getEntityId() + " of type " + eto.ek.getEntityTypeId());
                allTaggedEntities.remove(0);
                return;
            }
        }
        
        if (allTaggedEntities.size() > 1)
        {
            for (int i = 0; i < allTaggedEntities.size(); ++i)
            {
                boolean markCurrentEntityForDeletion = false;
                boolean foundAllowedEntityType = false;
                String deletionLogMessage = "";
                EntityTaggerOutput etoCurrent = allTaggedEntities.get(i);
                EntityKeyword ekCurrent = etoCurrent.ek;
                if (ekCurrent != null)
                {
                    Set<Long> allowedPostTypes = postAllowedEntityTypes.get(ekCurrent.getEntityTypeId());
                    if (allowedPostTypes != null)
                    {
                        if (i < (allTaggedEntities.size() - 1))
                        {
                            EntityTaggerOutput etoNext = allTaggedEntities.get(i + 1);
                            EntityKeyword ekNext = etoNext.ek;
                            if (ekNext != null)
                            {
                                TokenGroup tgCurrent = etoCurrent.tg;
                                TokenGroup tgNext = etoNext.tg;
                                if (tgCurrent != null && tgNext != null)
                                {
                                    int tibStart = tgCurrent.getIndex() + tgCurrent.getSize();
                                    int tibEnd = tgNext.getIndex();
                                    if (tibStart < tibEnd)
                                    {
                                        String tib = input.substring(tibStart, tibEnd).trim();
                                        if (tib.length() == 0 || tib.matches("\\W+") || allowedIntermediateText.contains(tib))
                                        {
                                            if (!allowedPostTypes.contains(ekNext.getEntityTypeId()))
                                            {
//                                                EntityTaggerOutput etoToDiscard = allTaggedEntities.get(i);
//                                                LOGGER.debug("Entity " + etoToDiscard.tg.getToken() + " at index " + etoToDiscard.tg.getIndex() + " fails Proximity Rule, discarding. {intermediate text:" + tib + "}");
//                                                allTaggedEntities.remove(i);
//                                                --i;
                                                markCurrentEntityForDeletion = true;
                                                deletionLogMessage = "Entity " + etoCurrent.tg.getToken() + " at index " + etoCurrent.tg.getIndex() + " fails Proximity Rule (post), discarding. {intermediate text:" + tib + "}";
                                            }
                                            else
                                            {
                                                foundAllowedEntityType = true;
                                            }
                                        }
                                        else
                                        {
//                                            EntityTaggerOutput etoToDiscard = allTaggedEntities.get(i);
//                                            LOGGER.debug("Entity " + etoToDiscard.tg.getToken() + " at index " + etoToDiscard.tg.getIndex() + " fails Proximity Rule (intermediate text not allowed), discarding. {intermediate text:" + tib + "}");
//                                            allTaggedEntities.remove(i);
//                                            --i;                                                                                
//                                            continue;
                                            markCurrentEntityForDeletion = true;
                                            deletionLogMessage = "Entity " + etoCurrent.tg.getToken() + " at index " + etoCurrent.tg.getIndex() + " fails Proximity Rule (post, intermediate text not allowed), discarding. {intermediate text:" + tib + "}";
                                        }
                                    }
                                }
                            }
                        }                    
                    }                
                    
                    Set<Long> allowedPreTypes = postAllowedEntityTypes.get(ekCurrent.getEntityTypeId());                    
                    if (!foundAllowedEntityType)
                    {
                        if (allowedPreTypes != null)
                        {
                            if (i >= 1)
                            {
                                EntityTaggerOutput etoPrev = allTaggedEntities.get(i - 1);
                                EntityKeyword ekPrev = etoPrev.ek;
                                if (ekPrev != null)
                                {
                                    TokenGroup tgCurrent = etoCurrent.tg;
                                    TokenGroup tgPrev = etoPrev.tg;
                                    if (tgCurrent != null && tgPrev != null)
                                    {
                                        int tibStart = tgPrev.getIndex() + tgPrev.getSize();
                                        int tibEnd = tgCurrent.getIndex();
                                        if (tibStart < tibEnd)
                                        {
                                            String tib = input.substring(tibStart, tibEnd).trim();
                                            if (tib.length() == 0 || tib.matches("\\W+") || allowedIntermediateText.contains(tib))
                                            {
                                                if (!allowedPreTypes.contains(ekPrev.getEntityTypeId()))
                                                {
    //                                                EntityTaggerOutput etoToDiscard = allTaggedEntities.get(i);
    //                                                LOGGER.debug("Entity " + etoToDiscard.tg.getToken() + " at index " + etoToDiscard.tg.getIndex() + " fails Proximity Rule (pre), discarding. {intermediate text:" + tib + "}");
    //                                                allTaggedEntities.remove(i);
    //                                                --i;
                                                    markCurrentEntityForDeletion = true;
                                                    deletionLogMessage = "Entity " + etoCurrent.tg.getToken() + " at index " + etoCurrent.tg.getIndex() + " fails Proximity Rule (pre), discarding. {intermediate text:" + tib + "} {prev ek:" + ekPrev.getKeyword() + ", entity type = " + ekPrev.getEntityTypeId() + "}";
                                                }
                                                else
                                                {
                                                    markCurrentEntityForDeletion = false;
                                                }
                                            }
                                            else 
                                            {
    //                                            EntityTaggerOutput etoToDiscard = allTaggedEntities.get(i);
    //                                            LOGGER.debug("Entity " + etoToDiscard.tg.getToken() + " at index " + etoToDiscard.tg.getIndex() + " fails Proximity Rule (pre, intermediate text not allowed), discarding. {intermediate text:" + tib + "}");                                                
    //                                            allTaggedEntities.remove(i);
    //                                            --i;
                                                markCurrentEntityForDeletion = true;
                                                deletionLogMessage = "Entity " + etoCurrent.tg.getToken() + " at index " + etoCurrent.tg.getIndex() + " fails Proximity Rule (pre, intermediate text not allowed), discarding. {intermediate text:" + tib + "}";
                                            }
                                        }
                                    }
                                }                        
                            }                        
                        }                    
                    }
                    
                    if (markCurrentEntityForDeletion)
                    {
                        LOGGER.debug(deletionLogMessage);
                        allTaggedEntities.remove(i);
                        --i;
                    }
                }
            }
        }
    }
    
}
