/*
 * This Java source file is completely Free and Open-Source, 
 * released under the MIT License.
 *
 * Copyright (2012) Ashish Nawal Saharia (ashish.saharia@yahoo.co.in)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to deal 
 * in the Software without restriction, including without limitation the rights 
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
 * OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 */

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package net.infantai.nlp.ner;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import net.infantai.util.common.CommonUtil;
import net.infantai.util.common.TextUtil;
import net.infantai.util.common.TokenGroup;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 *
 * @author Ashish (ashish.saharia@yahoo.co.in)
 */
public class ValueExtractor
{
    
    private static final Logger LOGGER = LogManager.getLogger(ValueExtractor.class);
    
    private static final String REGEX_FOR_WHITESPACE = "[ \r\n	    ]+";
    private static final String REGEX_FOR_VALUE_DELIMITERS = "[ \r\n	    ,%°()\\-{}\\[\\]]+";
    
//    private static final char[] NUMERIC_TOKEN_DELIM_CHARACTERS = new char[]{' ', '', '	', 13, 10, ':', '=', '-', '%', '°', 176, 248, '\'', '"', '’', '”'}; // do not include '.' and '/' here. ASCII code 176, 248 might be used as degree signs
    private static final char[] NUMERIC_TOKEN_DELIM_CHARACTERS = new char[]{' ', '', '	', 13, 10, ':', '=', '-', '%', '°', 176, 248, '\'', '"', '’', '”', '(', ')', '[', ']', '{', '}'}; // do not include '.' and '/' here. ASCII code 176, 248 might be used as degree signs
    private static final char[] numChars = new char[]{'0','1','2','3','4','5','6','7','8','9'};
    
    private static final TextUtil tu = new TextUtil(REGEX_FOR_VALUE_DELIMITERS);
    
//    public List<EntityTaggerOutput> mergeValuesAndModifyEntities(String input, List<EntityTaggerOutput> etoListSortedByIndex, Set<String> allowedPreTokens, Set<String> allowedPostTokens, Set<Long> entityTypesToModify)
//    {
//        List<EntityTaggerOutput> etoList = new ArrayList<>();
//        char[] charcatersToTrim = new char[] {',', ' ', '	'};
//        for (int i = 0; i < etoListSortedByIndex.size(); ++i)
//        {
//            EntityTaggerOutput eto = etoListSortedByIndex.get(i);
//            if (eto.ek != null && entityTypesToModify.contains(eto.ek.getEntityTypeId()))
//            {
//                // check next token first                
//                int nextDelimIndex = TextUtil.getNextIndexOfCharacter(input, eto.tg.getIndex(), new char[]{' ', '', '	', 13, 10, ':', '=', '-'});
//                if (nextDelimIndex != -1)
//                {
//                    String nextToken = tu.getNTokensFromNextText(input.substring(nextDelimIndex), 1);
//                    int nextTokenIndex = input.indexOf(nextToken, eto.tg.getIndex() + eto.tg.getSize());
//                    if (nextToken.matches("[0-9\\./]+"))
//                    {
//                        // experimental: merge with tagged entity - changes a fundamental property that matched dictionary entity would be an exact match of a keyword
//                        if ((eto.tg.getIndex() + eto.tg.getSize()) < input.length())
//                        {
//                            if (nextTokenIndex != -1)
//                            {
//                                String newToken = input.substring(eto.tg.getIndex(), nextTokenIndex + nextToken.length());
//                                LOG.debug("Merging value with Entity Keyword: " + eto.tg.getToken() + " => " + newToken);
//                                eto.tg.setToken(newToken);
//                                eto.tg.setSize(eto.tg.getToken().length());
//                            }
//                        }
//                    }
//                    else
//                    {
//                        nextDelimIndex = TextUtil.getNextIndexOfCharacter(input, eto.tg.getIndex(), new char[]{' ', '', '	', 13, 10, ':', '=', '-'});
//                        if (nextDelimIndex != -1)
//                        {
//                            String nextToken2 = tu.getNTokensFromNextText(input.substring(nextDelimIndex), 1);
//                            int nextToken2Index = input.indexOf(nextToken2, eto.tg.getIndex() + eto.tg.getSize());
//                            if (nextToken2.matches("[0-9./]+"))
//                            {
//                                // experimental: merge with tagged entity - changes a fundamental property that matched dictionary entity would be an exact match of a keyword
//                                if ((eto.tg.getIndex() + eto.tg.getSize()) < input.length())
//                                {
//                                    if (nextToken2Index != -1)
//                                    {
//                                        eto.tg.setToken(input.substring(eto.tg.getIndex(), nextToken2Index + nextToken2.length()));
//                                        eto.tg.setSize(eto.tg.getToken().length());
//                                    }
//                                }
//                            }                        
//                        }                    
//                    }                
//                }
//
//                
//            }
//            etoList.add(eto);
//        }
//        return etoList;
//    }    
    
    private EntityTaggerOutput getEntityTaggerOutputOfPreviousEntityWithLowerIndex(List<EntityTaggerOutput> etoListSortedByIndex, int currentETOIndexInList)
    {
        EntityTaggerOutput eto = null;
        EntityTaggerOutput etoCurrent = etoListSortedByIndex.get(currentETOIndexInList);
        for (int i = (currentETOIndexInList - 1); i >= 0; --i)
        {
            eto = etoListSortedByIndex.get(i);
            if (eto.tg.getIndex() < etoCurrent.tg.getIndex())
            {
                break;
            }
            else
            {
                eto = null;
            }
        }
        return eto;
    }
    
    public List<EntityTaggerOutput> mergeValuesAndModifyEntities(String input, List<EntityTaggerOutput> etoListSortedByIndex, Set<String> allowedPreTokens, Set<String> allowedPostTokens, Set<Long> entityTypesToModify)
    {
        List<EntityTaggerOutput> etoList = new ArrayList<>();
//        char[] charcatersToTrim = new char[] {',', ' ', '	'};
        for (int i = 0; i < etoListSortedByIndex.size(); ++i)
        {
            EntityTaggerOutput eto = etoListSortedByIndex.get(i);
            if (eto.ek != null && entityTypesToModify.contains(eto.ek.getEntityTypeId()))
            {
                // check next token first                
                int nextDelimIndex = TextUtil.getNextIndexOfCharacter(input, (eto.tg.getIndex() + eto.tg.getSize()), NUMERIC_TOKEN_DELIM_CHARACTERS);
                if (nextDelimIndex != -1)
                {                    
//                    int nextDigitIndex = TextUtil.getNextIndexOfCharacter(input, eto.tg.getIndex(), numChars);
                    int nextDigitIndex = TextUtil.getNextIndexOfCharacter(input, nextDelimIndex, numChars);
                    
                    if (nextDigitIndex != -1)
                    {
                        String numericValueToken = tu.getNTokensFromNextText(input.substring(nextDigitIndex), 1);
                        int currentETOEndIndex = eto.tg.getIndex() + eto.tg.getSize();
                        String tib = input.substring(currentETOEndIndex, nextDigitIndex).trim();
                        if (tib.length() == 0 || allowedPostTokens.contains(tib) || tib.matches("\\W+"))
                        {
//                            int nextTokenIndex = input.indexOf(numericValueToken, eto.tg.getIndex() + eto.tg.getSize());
                            if (numericValueToken.matches("[0-9\\./]+"))
                            {
                                if (nextDigitIndex != -1)
                                {
                                    if ((nextDigitIndex + numericValueToken.length()) <= input.length())                                        
                                    {
//                                        // experimental: merge with tagged entity - changes a fundamental property that matched dictionary entity would be an exact match of a keyword
//                                        String newToken = input.substring(eto.tg.getIndex(), nextDigitIndex + numericValueToken.length());
//                                        LOG.debug("Merging value with Entity Keyword: " + eto.tg.getToken() + " => " + newToken);
//                                        eto.tg.setToken(newToken);
//                                        eto.tg.setSize(eto.tg.getToken().length());
                                        eto.possibleValue = numericValueToken;
                                        eto.possibleValueIndex = nextDigitIndex;
                                        LOGGER.debug("Found possible value with Entity Keyword: " + eto.tg.getToken() + " {index: " + eto.tg.getIndex() + "}" + " => " + eto.possibleValue);                                        
                                        etoList.add(eto);
                                        continue;
                                    }
                                }
                            }                            
                        }
                    }                
                }
                
                // check prev token                
                int prevDelimIndex = TextUtil.getPrevIndexOfCharacter(input, eto.tg.getIndex(), NUMERIC_TOKEN_DELIM_CHARACTERS);
                if (prevDelimIndex != -1)
                {                    
                    int prevDigitIndex = TextUtil.getPrevIndexOfCharacter(input, eto.tg.getIndex(), numChars); // from 38.7, this gets to 7
                    
                    if (prevDigitIndex != -1)
                    {
                        
                        int prevNumIndex = TextUtil.getPrevIndexOfCharacter(input, prevDigitIndex, NUMERIC_TOKEN_DELIM_CHARACTERS);
                        if (prevNumIndex == -1)
                        {
                            prevNumIndex = 0;
                        }
                        else
                        {
                            prevNumIndex = prevNumIndex + 1;
                        }
                        
                        int prevNumEndIndex = prevDigitIndex + 1;
                        String numericValueToken = tu.getNTokensFromPrevText(input.substring(0, prevNumEndIndex), 1);
                        int tibStart = prevNumEndIndex;
                        if (tibStart < eto.tg.getIndex() && tibStart < input.length())
                        {
                            String tib = input.substring(tibStart, eto.tg.getIndex()).trim();
                            if (tib.length() == 0 || allowedPreTokens.contains(tib) || tib.matches("\\W+"))
                            {
    //                            int prevTokenIndex = input.indexOf(numericValueToken, eto.tg.getIndex() + eto.tg.getSize());
                                if (numericValueToken.matches("[0-9\\./]+"))
                                {
                                    if (prevDigitIndex != -1)
                                    {
                                        EntityTaggerOutput etoLeft = getEntityTaggerOutputOfPreviousEntityWithLowerIndex(etoListSortedByIndex, i);
                                        if (etoLeft != null)
                                        {
                                            if (etoLeft.tg.getIndex() + etoLeft.tg.getSize() < prevNumIndex) // don't allow overlaps
                                            {
                                                if ((prevNumIndex + numericValueToken.length()) <= input.length())                                        
                                                {
            //                                        // experimental: merge with tagged entity - changes a fundamental property that matched dictionary entity would be an exact match of a keyword
            //                                        String newToken = input.substring(prevNumIndex, eto.tg.getIndex() + eto.tg.getToken().length());
            //                                        LOG.debug("Merging value with Entity Keyword: " + eto.tg.getToken() + " => " + newToken);
            //                                        eto.tg.setToken(newToken);
            //                                        eto.tg.setIndex(prevNumIndex);
            //                                        eto.tg.setSize(eto.tg.getToken().length());
                                                    eto.possibleValue = numericValueToken;
                                                    eto.possibleValueIndex = prevNumIndex;
                                                    LOGGER.debug("Found possible value with Entity Keyword: " + eto.tg.getToken() + " {index: " + eto.tg.getIndex() + "}" + " => " + eto.possibleValue + ", value index: " + eto.possibleValueIndex);
                                                }                                        
                                            }                                        
                                        }
                                        else
                                        {
                                            if ((prevNumIndex + numericValueToken.length()) <= input.length())                                        
                                            {
        //                                        // experimental: merge with tagged entity - changes a fundamental property that matched dictionary entity would be an exact match of a keyword
        //                                        String newToken = input.substring(prevNumIndex, eto.tg.getIndex() + eto.tg.getToken().length());
        //                                        LOG.debug("Merging value with Entity Keyword: " + eto.tg.getToken() + " => " + newToken);
        //                                        eto.tg.setToken(newToken);
        //                                        eto.tg.setIndex(prevNumIndex);
        //                                        eto.tg.setSize(eto.tg.getToken().length());
                                                eto.possibleValue = numericValueToken;
                                                eto.possibleValueIndex = prevNumIndex;
                                                LOGGER.debug("Found possible value with Entity Keyword: " + eto.tg.getToken() + " {index: " + eto.tg.getIndex() + "}" + " => " + eto.possibleValue + ", value index: " + eto.possibleValueIndex);
                                            }
                                        }
                                    }
                                }                            
                            }                        
                        }
                        else
                        {
                            LOGGER.debug("Incorrect index: " + input + ", start: " + tibStart + ", tg: " + eto.tg + ", tg index: " + eto.tg.getIndex() + ", numeric value: " + numericValueToken + ", prevDigitIndex: " + prevDigitIndex);
                        }
                    }                
                }                
                
            }
            etoList.add(eto);
        }
        return etoList;
    }

    public static void main(String[] args)
    {
        ValueExtractor ve = new ValueExtractor();
        String input = "Blood Pressure [158/88]";
        TokenGroup tg = new TokenGroup("Blood Pressure", 0, "Blood Pressure".length());
        List<EntityTaggerOutput> etoListSortedByIndex = new ArrayList<>();
        EntityTaggerOutput eto = new EntityTaggerOutput();
        EntityKeyword ek = new EntityKeyword();
        ek.setEntityTypeId(99L);
        eto.ek = ek;
        eto.tg = tg;
        etoListSortedByIndex.add(eto);
        Set<String> allowedPreTokens = new HashSet<>();
        Set<String> allowedPostTokens = new HashSet<>();
        Set<Long> entityTypesToModify = new HashSet<>();
        entityTypesToModify.add(99L);
        ve.mergeValuesAndModifyEntities(input, etoListSortedByIndex, allowedPreTokens, allowedPostTokens, entityTypesToModify);
//        String[] splitTest = input.split(REGEX_FOR_VALUE_DELIMITERS);
//        for (int i = 0; i < splitTest.length; ++i)
//        {
//            System.out.println(splitTest[i]);
//        }
    }
    
}
