/*
 * This Java source file is completely Free and Open-Source, 
 * released under the MIT License.
 *
 * Copyright (2012) Ashish Nawal Saharia (ashish.saharia@yahoo.co.in)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to deal 
 * in the Software without restriction, including without limitation the rights 
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
 * OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 */

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package net.infantai.util.common;

import java.io.UnsupportedEncodingException;

/**
 *
 * A class to convert numbers to byte arrays.
 * 
 * @author Ashish (ashish.saharia@yahoo.co.in)
 */
public class ByteConverter
{
    
    public static byte[] toByteArray(short value)
    {
        byte[] ba = new byte[2];
        ba[0] = (byte) (value >> 8);
        ba[1] = (byte) (value);
        return ba;
    }    
    
    public static byte[] toByteArray(int value)
    {
        byte[] ba = new byte[4];
        ba[0] = (byte) (value >> 24);
        ba[1] = (byte) (value >> 16);
        ba[2] = (byte) (value >> 8);
        ba[3] = (byte) (value);
        return ba;
    }
    
    public static byte[] toByteArray(long value)
    {
        byte[] ba = new byte[8];
        ba[0] = (byte) (value >> 56);
        ba[1] = (byte) (value >> 48);
        ba[2] = (byte) (value >> 40);
        ba[3] = (byte) (value >> 32);
        ba[4] = (byte) (value >> 24);
        ba[5] = (byte) (value >> 16);
        ba[6] = (byte) (value >> 8);
        ba[7] = (byte) (value);
        return ba;
    }
    
    public static byte[] toByteArray(float value)
    {
        return toByteArray(Float.floatToRawIntBits(value));
    }    
    
    public static byte[] toByteArray(double value)
    {
        return toByteArray(Double.doubleToRawLongBits(value));
    }
    
    /**
     * <b>WARNING: </b> Do <b>not</b> use this method with strings containing multi-byte characters.
     * <br>
     * Returns a byte representation of the input String. This method differs 
     * from the getBytes() method (and its overloaded forms) of the String class, 
     * as it does not rely on any Character Encoding to interpret the input String. 
     * It is designed as a convenience method to be used <b>only</b> with strings 
     * comprised entirely of single byte (8-bit) characters. For any other character 
     * sets, use the <i>getBytes(Charset charset)</i> method of the String class. 
     * <br>
     * A char in Java is a 2-byte (unsigned) data type. For each character in the 
     * input string, this method will completely ignore the left byte, and place the 
     * right byte in an array. This method is slightly faster than 
     * <i>getBytes(Charset charset)</i> from the String class.
     * <br>
     * This method will throw an UnsupportedEncodingException on a best-effort 
     * basis, if a multi-byte character is found in the input string. 
     * <br>
     * <b>WARNING: </b> Do <b>not</b> use this method with strings containing multi-byte characters.
     * @param input A String with only ASCII / ISO-8859-1 characters.
     * @return A byte representation of the input String.
     */
    public static byte[] toByteArray(String input) throws UnsupportedEncodingException
    {
        byte[] ba = new byte[input.length()];
        for (int i = 0; i < input.length(); ++i)
        {
            char ch = input.charAt(i);
            byte leftByte = (byte) ((ch & 0xFF00) >> 8);
            if (leftByte != (byte) 0)
            {
                throw new UnsupportedEncodingException("Only single-byte encodings allowed; found unsupported character (" + ch + ") at index " + i + ".");
            }
            ba[i] = (byte) (ch & 0x00FF);
        }
        return ba;
    }
    
}
