/*
 * This Java source file is completely Free and Open-Source, 
 * released under the MIT License.
 *
 * Copyright (2012) Ashish Nawal Saharia (ashish.saharia@yahoo.co.in)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to deal 
 * in the Software without restriction, including without limitation the rights 
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
 * OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 */

/*
 * DateFinder.java
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */
package net.infantai.util.common;

import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 *
 * @author Ashish (ashish.saharia@yahoo.co.in)
 */
public class DateFinder
{

    private static DateFinder dateFinder = null;

    private static Pattern patLiteral;
    private static Pattern patRelative;

    private static Pattern patDDMMYYYY;
    private static Pattern patYYYYMMDD;
    private static Pattern patInYYYY;
    private static Pattern patNextWeekMonthYear;
    private static Pattern patDayMonthYear01;
    private static Pattern patMonthYear01;
    private static Pattern patMonthDayYear01;
    private static Pattern patTomorrow;
    private static Pattern patNextNYears;
    private static Pattern patNYearsFromNow;
    private static Pattern patMonthOnly;
    private static Pattern patMonthAndDay;

    private static final Logger LOGGER = LogManager.getLogger(DateFinder.class);

    private static final Object singletonLock = new Object();

    private DateFinder()
    {
        initializePatterns();
    }

    public static DateFinder getInstance()
    {
        synchronized (singletonLock)
        {
            if (dateFinder == null)
            {
                dateFinder = new DateFinder();
            }
        }
        return dateFinder;
    }

    private static void initializePatterns()
    {

//        String regexpLiteral1 = "(\\A|\\W+|\\s+)(([0-9]{4}(/|-|\\.)([0123]?[0-9]{1})(/|-|\\.)([0123]?[0-9]{1}))|((([0123]?[0-9]{1})(st|nd|rd|th)?\\s+(January|February|March|April|May|June|July|August|September|October|November|December|Jan\\.?|Feb\\.?|Mar\\.?|Apr\\.?|May|Jun\\.?|Jul\\.?|Aug\\.?|Sep\\.?|Oct\\.?|Nov\\.?|Dec\\.?),?(\\s+([0-9]{4}|'?[0-9]{2}))?)|(January|February|March|April|May|June|July|August|September|October|November|December|Jan\\.?|Feb\\.?|Mar\\.?|Apr\\.?|May|Jun\\.?|Jul\\.?|Aug\\.?|Sep\\.?|Oct\\.?|Nov\\.?|Dec\\.?)(\\s+([0123]?[0-9]{1})(st|nd|rd|th)?)?,?\\s+([0-9]{4}|'?[0-9]{2})?)|(([0123]?[0-9]{1})(/|-|\\.)([0123]?[0-9]{1})(/|-|\\.)([0-9]{4}|[0-9]{2}))|([0-9]{2}(/|-|\\.)([0123]?[0-9]{1})(/|-|\\.)([0123]?[0-9]{1}))|(january|february|march|april|may|june|july|august|september|october|november|december|jan\\.?|feb\\.?|mar\\.?|apr\\.?|may|jun\\.?|jul\\.?|aug\\.?|sep\\.?|oct\\.?|nov\\.?|dec\\.?),?(\\s+([0-9]{4}|'?[0-9]{2})))(\\s+|\\W+|\\z)";
        String regexpLiteral = "(\\A|\\W+|\\s+)(([0-9]{4}(/|-|\\.)([0123]?[0-9]{1})(/|-|\\.)([0123]?[0-9]{1}))|((([0123]?[0-9]{1})(st|nd|rd|th)?\\s+(January|February|March|April|May|June|July|August|September|October|November|December|Jan\\.?|Feb\\.?|Mar\\.?|Apr\\.?|May|Jun\\.?|Jul\\.?|Aug\\.?|Sep\\.?|Oct\\.?|Nov\\.?|Dec\\.?),?(\\s+([0-9]{4}|'?[0-9]{2}))?)|(January|February|March|April|May|June|July|August|September|October|November|December|Jan\\.?|Feb\\.?|Mar\\.?|Apr\\.?|May|Jun\\.?|Jul\\.?|Aug\\.?|Sep\\.?|Oct\\.?|Nov\\.?|Dec\\.?)[ -/]+[0-9]{1,2}[ -/,]+[0-9]{1,2}\\:[0-9]{2}\\s*(?:am|pm)?|(January|February|March|April|May|June|July|August|September|October|November|December|Jan\\.?|Feb\\.?|Mar\\.?|Apr\\.?|May|Jun\\.?|Jul\\.?|Aug\\.?|Sep\\.?|Oct\\.?|Nov\\.?|Dec\\.?)(\\s+([0123]?[0-9]{1})(st|nd|rd|th)?)?,?\\s+([0-9]{4}|'?[0-9]{2})?)|(([0123]?[0-9]{1})(/|-|\\.)([0123]?[0-9]{1})(/|-|\\.)([0-9]{4}|[0-9]{2}))|([0-9]{2}(/|-|\\.)([0123]?[0-9]{1})(/|-|\\.)([0123]?[0-9]{1}))|(january|february|march|april|may|june|july|august|september|october|november|december|jan\\.?|feb\\.?|mar\\.?|apr\\.?|may|jun\\.?|jul\\.?|aug\\.?|sep\\.?|oct\\.?|nov\\.?|dec\\.?),?(\\s+([0-9]{4}|'?[0-9]{2})))(\\s+|\\W+|\\z)"; //working
        
        patLiteral = Pattern.compile(regexpLiteral, Pattern.CASE_INSENSITIVE | Pattern.DOTALL | Pattern.MULTILINE);

//        regexRelative = "(\\A|\\W+|\\s+)(strTomorrow|next week|next month|(next ((one|two|three|four|five|six|seven|eight|nine|ten|1|2|3|4|5|6|7|8|9|10) )?year(s)?)|(next ((one|two|three|four|five|six|seven|eight|nine|ten|1|2|3|4|5|6|7|8|9|10) )?months(s)?)|(next ((one|two|three|four|five|six|seven|eight|nine|ten|1|2|3|4|5|6|7|8|9|10) )?week(s)?)|(next ((one|two|three|four|five|six|seven|eight|nine|ten|1|2|3|4|5|6|7|8|9|10) )?quarter(s)?)|(((one|two|three|four|five|six|seven|eight|nine|ten|1|2|3|4|5|6|7|8|9|10) )((quarter(s)?)|(years(s)?)|(months(s)?)|(week(s)?))(( from now)|( later)))|((in|until|to|scheduled for|set for|year) [0-9]{4}))(\\s+|\\W+|\\z)";
        String regexRelative = "(\\A|\\W+|\\s+)(tomorrow|(next ((one|two|three|four|five|six|seven|eight|nine|ten|1|2|3|4|5|6|7|8|9|10) )?(((year|years)?)|((quarter|quarters)?)|((month|months)?)|((week|weeks)?)))|(((one|two|three|four|five|six|seven|eight|nine|ten|1|2|3|4|5|6|7|8|9|10) )((years(s)?)|(quarter(s)?)|(months(s)?)|(week(s)?))(( from now)|( later)))|((in|until|till|to|scheduled for|set for|year) [0-9]{4}))(\\s+|\\W+|\\z)";
        patRelative = Pattern.compile(regexRelative, Pattern.CASE_INSENSITIVE | Pattern.DOTALL | Pattern.MULTILINE);

        String strPatDDMMYYYY = "(?:\\A|\\W+)\\d{1,2}[.\\/-]{1}\\d{1,2}[.\\/-]{1}((\\d{2})|(\\d{4}))";
        String strPatYYYYMMDD = "(?:\\A|\\W+)\\d{4}[.\\/-]{1}\\d{1,2}[.\\/-]{1}\\d{1,2}";
        String strPatInYYYY = "((in|until|till|to|scheduled for|set for|year) [0-9]{4})";
        String strPatNextWeekMonthYear = "next (week|month|year|quarter)";
        String strPatDayMonthYear01 = "(([0123]?[0-9]{1})(st|nd|rd|th)?\\s+(january|february|march|april|may|june|july|august|september|october|november|december|jan\\.?|feb\\.?|mar\\.?|apr\\.?|may|jun\\.?|jul\\.?|aug\\.?|sep\\.?|oct\\.?|nov\\.?|dec\\.?),?(\\s+([0-9]{4}|'?[0-9]{2}))?)";
        String strPatMonthYear01 = "((?:\\A|\\W+)(january|february|march|april|may|june|july|august|september|october|november|december|jan\\.?|feb\\.?|mar\\.?|apr\\.?|may|jun\\.?|jul\\.?|aug\\.?|sep\\.?|oct\\.?|nov\\.?|dec\\.?),?(\\s+([0-9]{4}|'?[0-9]{2}))?)";
        String strPatMonthDayYear01 = "((january|february|march|april|may|june|july|august|september|october|november|december|jan\\.?|feb\\.?|mar\\.?|apr\\.?|may|jun\\.?|jul\\.?|aug\\.?|sep\\.?|oct\\.?|nov\\.?|dec\\.?)(\\s+([0123]?[0-9]{1})(st|nd|rd|th)?)?,?\\s+([0-9]{4}|'?[0-9]{2})?)";
        String strTomorrow = "tomorrow";
        String strNextNYears = "(next ((one|two|three|four|five|six|seven|eight|nine|ten|1|2|3|4|5|6|7|8|9|10) )(((year|years)?)|((quarter|quarters)?)|((month|months)?)|((week|weeks)?)))";
        String strNYearsFromNow = "(((one|two|three|four|five|six|seven|eight|nine|ten|1|2|3|4|5|6|7|8|9|10) )((years(s)?)|(quarter(s)?)|(months(s)?)|(week(s)?))(( from now)|( later)))";
        String strMonthOnly = "(january|february|march|april|may|june|july|august|september|october|november|december|jan\\.?|feb\\.?|mar\\.?|apr\\.?|may|jun\\.?|jul\\.?|aug\\.?|sep\\.?|oct\\.?|nov\\.?|dec\\.?)";
//        String strMonthAndDay = "((january|february|march|april|may|june|july|august|september|october|november|december|jan\\.?|feb\\.?|mar\\.?|apr\\.?|may|jun\\.?|jul\\.?|aug\\.?|sep\\.?|oct\\.?|nov\\.?|dec\\.?)(\\s+([0123]?[0-9]{1})(st|nd|rd|th)?))";
        String strMonthAndDay = "((january|february|march|april|may|june|july|august|september|october|november|december|jan\\.?|feb\\.?|mar\\.?|apr\\.?|may|jun\\.?|jul\\.?|aug\\.?|sep\\.?|oct\\.?|nov\\.?|dec\\.?)(\\s+([0123]?[0-9]{1})(st|nd|rd|th)?)(?: [0-9]{1,2}\\:[0-9]{2}\\s*(?:am|pm)?)?)";

        patDDMMYYYY = Pattern.compile(strPatDDMMYYYY, Pattern.CASE_INSENSITIVE | Pattern.DOTALL | Pattern.MULTILINE);
        patYYYYMMDD = Pattern.compile(strPatYYYYMMDD, Pattern.CASE_INSENSITIVE | Pattern.DOTALL | Pattern.MULTILINE);
        patInYYYY = Pattern.compile(strPatInYYYY, Pattern.CASE_INSENSITIVE | Pattern.DOTALL | Pattern.MULTILINE);
        patNextWeekMonthYear = Pattern.compile(strPatNextWeekMonthYear, Pattern.CASE_INSENSITIVE | Pattern.DOTALL | Pattern.MULTILINE);
        patDayMonthYear01 = Pattern.compile(strPatDayMonthYear01, Pattern.CASE_INSENSITIVE | Pattern.DOTALL | Pattern.MULTILINE);
        patMonthYear01 = Pattern.compile(strPatMonthYear01, Pattern.CASE_INSENSITIVE | Pattern.DOTALL | Pattern.MULTILINE);
        patMonthDayYear01 = Pattern.compile(strPatMonthDayYear01, Pattern.CASE_INSENSITIVE | Pattern.DOTALL | Pattern.MULTILINE);
        patTomorrow = Pattern.compile(strTomorrow, Pattern.CASE_INSENSITIVE | Pattern.DOTALL | Pattern.MULTILINE);
        patNextNYears = Pattern.compile(strNextNYears, Pattern.CASE_INSENSITIVE | Pattern.DOTALL | Pattern.MULTILINE);
        patNYearsFromNow = Pattern.compile(strNYearsFromNow, Pattern.CASE_INSENSITIVE | Pattern.DOTALL | Pattern.MULTILINE);
        patMonthOnly = Pattern.compile(strMonthOnly, Pattern.CASE_INSENSITIVE | Pattern.DOTALL | Pattern.MULTILINE);
        patMonthAndDay = Pattern.compile(strMonthAndDay, Pattern.CASE_INSENSITIVE | Pattern.DOTALL | Pattern.MULTILINE);

    }

    public static void main(String[] args)
    {
        DateFinder df = DateFinder.getInstance();
//        String inputText = "20 November 2454 FORT Jan 15th, 2009 Augustine 21 Feb 2004 and 08/07/09 LAUDERDALE, Fla., March 2013 /PRNewswire-FirstCall/ 03 August 1981 -- Magnum D'Or Resources, Inc. As Aug. 03, 2009 on 18/3/2008 or maybe, 1 March, 2009 March 21, 2007 and 20th Dec. what about 08/03/1981 or 2008/08/03 ? And is something is scheduled for next one year? or next month. or even tomorrow, or next week, or next year, or perhaps the next 4 months, or the next five weeks? And something that happens four months later ? 8 months from now ? five years from now, something cool will happen.";
//        String inputText = "For the six months to 31 December 2011, TelstraClear posted a pre-tax profit rise of 11 per cent to ";
        String inputText = "NAME: MCGRATH, DAVID H. NUMBER:\n" +
"ADMITTED: 9-16-92 DISCHARGED: 9-27-92\n" +
"ATTENDING \\PHYSICIAN: M.D. "
                + "Fever 102 - NOV 23 07:20:59.205999"
                + "24th NOV 07:20 am \n" +
"23rd NOV 06:20";
        Map<Integer, String> alldates = df.getDateStrings(inputText);
        Iterator<Map.Entry<Integer, String>> iter = alldates.entrySet().iterator();
        while (iter.hasNext())
        {
            Map.Entry<Integer, String> mapentry = iter.next();
            Integer index = mapentry.getKey();
            String matchedString = mapentry.getValue();
//            DateParts dateparts = df.interpretDateString(355602652000L, matchedString);
            DateParts dateparts = df.interpretDateString(System.currentTimeMillis(), matchedString);
            LOGGER.debug("Found Match at index: " + index + ", Matched String: \"" + matchedString + "\" {DateParts:" + dateparts + "}");
            System.out.println(df.discardTimePartFromDatetimeString(matchedString));
        }
        System.out.println("--- " + df.discardTimePartFromDatetimeString("NOV 23 06:20:59.205999"));
    }

    public Map<Integer, String> getDateStrings(String content)
    {
        Map<Integer, String> dates = new LinkedHashMap<Integer, String>();

        if (content != null)
        {
            Matcher matLiteral = patLiteral.matcher(content);
            int i = 0;
            while (matLiteral.find())
            {
                if (matLiteral.groupCount() > 1)
                {
                    String date = matLiteral.group(2);
                    dates.put(new Integer(matLiteral.start(2)), date);
//                    LOG.debug((i < 9 ? "0" + ++i : ++i) + ". " + date);            
                }
            }

            Matcher matRelative = patRelative.matcher(content);
            while (matRelative.find())
            {
                if (matRelative.groupCount() > 1)
                {
                    String date = matRelative.group(2);
                    dates.put(new Integer(matRelative.start(2)), date);
//                    LOG.debug((i < 9 ? "0" + ++i : ++i) + ". " + date);            
                }
            }
        }
        return dates;
    }
    
    public String discardTimePartFromDatetimeString(String datetime)
    {
        return datetime.replaceAll("[ -/]+[0-9]{1,2}\\:[0-9]{2}(?:\\:[0-9]{2}(?:\\.[0-9]+)?)?\\s*(?:am|pm)?", "").trim();
    }

    public DateParts interpretDateString(long baseDate, String dateStr)
    {
        DateParts dateParts = null;
        char[] unwantedChar = new char[]{','};
        int baseYear = -1;
        int baseMonth = -1;
        int baseDay = -1;
        GregorianCalendar baseGregCal = new GregorianCalendar();

        CommonUtil genutil = new CommonUtil();

        dateStr = dateStr.toLowerCase().trim();
        dateStr = genutil.trimCharacters(unwantedChar, dateStr);

        if (baseDate >= 0)
        {
            baseGregCal.setTimeInMillis(baseDate);
        }
        
        baseYear = baseGregCal.get(GregorianCalendar.YEAR);
        baseMonth = baseGregCal.get(GregorianCalendar.MONTH) + 1; // zero based months
        baseDay = baseGregCal.get(GregorianCalendar.DAY_OF_MONTH);

        try
        {
            if (patDDMMYYYY.matcher(dateStr).find())
            {
                String[] strDateParts = dateStr.split("[.\\/-]+");
                int dd = Integer.parseInt(strDateParts[0]);
                int mm = Integer.parseInt(strDateParts[1]);
                String strYYYY = strDateParts[2];
                if (strYYYY.length() == 2)
                {
                    strYYYY = ("" + baseYear).substring(0, 2) + strDateParts[2];
                }

                int yyyy = Integer.parseInt(strYYYY);
                if (dd < 1 || dd > 31 || mm < 1 || mm > 31)
                {
                    return null; // not a date
                }

                if (mm > 12)
                {
                    int temp = dd;
                    dd = mm;
                    mm = temp;
                }

                if (mm > 12)
                {
                    return null; // not a date
                }

                dateParts = new DateParts();
                dateParts.setDD(dd);
                dateParts.setMM(mm);
                dateParts.setYYYY(yyyy);
            }
            else
            {
                if (patYYYYMMDD.matcher(dateStr).find())
                {
                    String[] strDateParts = dateStr.split("[.\\/-]");
                    int yyyy = Integer.parseInt(strDateParts[0]);
                    int mm = Integer.parseInt(strDateParts[1]);
                    int dd = Integer.parseInt(strDateParts[2]);
                    if (dd < 1 || dd > 31 || mm < 1 || mm > 31)
                    {
                        return null; // not a date
                    }

                    if (mm > 12)
                    {
                        int temp = dd;
                        dd = mm;
                        mm = temp;
                    }

                    if (mm > 12)
                    {
                        return null; // not a date
                    }

                    dateParts = new DateParts();
                    dateParts.setDD(dd);
                    dateParts.setMM(mm);
                    dateParts.setYYYY(yyyy);
                }
                else
                {
                    if (patInYYYY.matcher(dateStr).find())
                    {
                        String[] strDateParts = dateStr.split(" ");
                        int yyyy = Integer.parseInt(strDateParts[(strDateParts.length - 1)]);
                        dateParts = new DateParts();
                        dateParts.setYYYY(yyyy);
                    }
                    else
                    {
                        if (patNextWeekMonthYear.matcher(dateStr).find())
                        {
                            String[] strDateParts = dateStr.split("\\s+");
                            if (strDateParts[1].equals("year"))
                            {
                                dateParts = new DateParts();
                                dateParts.setYYYY(baseYear + 1);
                            }
                            else
                            {
                                if (strDateParts[1].equals("month"))
                                {
                                    dateParts = new DateParts();
                                    if (baseMonth == 12)
                                    {
                                        dateParts.setMM(1);
                                        dateParts.setYYYY(baseYear + 1);
                                    }
                                    else
                                    {
                                        dateParts.setMM(baseMonth + 1);
                                        dateParts.setYYYY(baseYear);
                                    }
                                }
                                else
                                {
                                    if (strDateParts[1].equals("month"))
                                    {
                                        dateParts = new DateParts();
                                        if (baseMonth == 12)
                                        {
                                            dateParts.setMM(1);
                                            dateParts.setYYYY(baseYear + 1);
                                        }
                                        else
                                        {
                                            dateParts.setMM(baseMonth + 1);
                                            dateParts.setYYYY(baseYear);
                                        }
                                    }
                                    else
                                    {
                                        if (strDateParts[1].equals("week"))
                                        {
                                            dateParts = new DateParts();
                                            if (baseDay >= 25)
                                            {
                                                if (baseMonth == 12)
                                                {
                                                    dateParts.setMM(1);
                                                    dateParts.setYYYY(baseYear + 1);
                                                }
                                                else
                                                {
                                                    dateParts.setMM(baseMonth + 1);
                                                    dateParts.setYYYY(baseYear);
                                                }
                                            }
                                            else
                                            {
                                                dateParts.setMM(baseMonth);
                                                dateParts.setYYYY(baseYear);
                                            }
                                        }
                                        else
                                        {
                                            if (strDateParts[1].equals("quarter"))
                                            {
                                                dateParts = new DateParts();
                                                if (baseMonth >= 10)
                                                {
                                                    dateParts.setMM(baseMonth - 9);
                                                    dateParts.setYYYY(baseYear + 1);
                                                }
                                                else
                                                {
                                                    dateParts.setMM(baseMonth + 3);
                                                    dateParts.setYYYY(baseYear);
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        else
                        {
                            if (patDayMonthYear01.matcher(dateStr).find())
                            {
                                String[] strDateParts = dateStr.split("\\s+");
                                if (strDateParts.length == 3)
                                {
                                    if (strDateParts[2].length() == 4) // not handling '12 for now...
                                    {
                                        dateParts = new DateParts();
                                        dateParts.setYYYY(Integer.parseInt(strDateParts[2]));

                                        int monthNumber = getMonthNumber(strDateParts[1]);
                                        if (monthNumber != -1)
                                        {
                                            dateParts.setMM(monthNumber);

                                            String day = strDateParts[0];
                                            int dayNumber = getDayNumber(day);
                                            if (dayNumber != -1)
                                            {
                                                dateParts.setDD(dayNumber);
                                            }
                                        }
                                    }
                                }
                                else
                                {
                                    if (strDateParts.length == 2)
                                    {
                                        if (strDateParts[0].length() <= 2 || (strDateParts[0].length() <= 4 && (strDateParts[0].endsWith("st") || strDateParts[0].endsWith("nd") || strDateParts[0].endsWith("rd") || strDateParts[0].endsWith("th"))))
                                        {
                                            dateParts = new DateParts();
                                            dateParts.setYYYY(baseYear);
                                            int monthNumber = getMonthNumber(strDateParts[1]);
                                            if (monthNumber != -1)
                                            {
                                                dateParts.setMM(monthNumber);

                                                int dayNumber = getDayNumber(strDateParts[0]);
                                                if (dayNumber != -1)
                                                {
                                                    dateParts.setDD(dayNumber);
                                                }
                                            }
                                        }
                                        else
                                        {
                                            LOGGER.error("Not Matched #1: " + dateStr);
                                        }
                                    }
                                }
                            }
                            else if (patMonthYear01.matcher(dateStr).find())
                            {
                                String[] strDateParts = dateStr.split("\\s+");
                                if (strDateParts.length == 2)
                                {
                                    if (strDateParts[1].length() == 4) // not handling '12 for now...
                                    {
                                        dateParts = new DateParts();
                                        dateParts.setYYYY(Integer.parseInt(strDateParts[1]));

                                        int monthNumber = getMonthNumber(strDateParts[0]);
                                        if (monthNumber != -1)
                                        {
                                            dateParts.setMM(monthNumber);
                                            
                                            // no day in date, only moth and year
                                        }
                                    }
                                }
                                else if (patMonthAndDay.matcher(dateStr).find()) // experimental
                                {
                                    if (strDateParts.length >= 3)
                                    {
                                        int monthNumber = getMonthNumber(strDateParts[0]);
                                        if (monthNumber != -1)
                                        {
                                            int dayNumber = getDayNumber(strDateParts[1]);
                                            if (dayNumber != -1)
                                            {
                                                dateParts = new DateParts();
                                                dateParts.setYYYY(baseYear);                                            
                                                dateParts.setMM(monthNumber);                                                
                                                dateParts.setDD(dayNumber);
                                            }                                            
                                        }                                        
                                    }
                                }
                            }                            
                            else
                            {
                                if (patMonthDayYear01.matcher(dateStr).find())
                                {
                                    String[] strDateParts = dateStr.split("\\s+");
                                    if (strDateParts.length == 3)
                                    {
                                        if (strDateParts[2].length() == 4) // not handling '12 for now...
                                        {
                                            dateParts = new DateParts();
                                            dateParts.setYYYY(Integer.parseInt(strDateParts[2]));

                                            int monthNumber = getMonthNumber(strDateParts[0]);
                                            if (monthNumber != -1)
                                            {
                                                dateParts.setMM(monthNumber);

                                                String day = strDateParts[1];
                                                int dayNumber = getDayNumber(day);
                                                if (dayNumber != -1)
                                                {
                                                    dateParts.setDD(dayNumber);
                                                }
                                            }
                                        }
                                    }
                                    else
                                    {
                                        if (strDateParts.length == 2)
                                        {
                                            if (strDateParts[1].length() <= 2 || (strDateParts[1].length() <= 4 && (strDateParts[1].endsWith("st") || strDateParts[1].endsWith("nd") || strDateParts[1].endsWith("rd") || strDateParts[1].endsWith("th"))))
                                            {
                                                dateParts = new DateParts();
                                                dateParts.setYYYY(baseYear);
                                                int monthNumber = getMonthNumber(strDateParts[0]);
                                                if (monthNumber != -1)
                                                {
                                                    dateParts.setMM(monthNumber);

                                                    int dayNumber = getDayNumber(strDateParts[1]);
                                                    if (dayNumber != -1)
                                                    {
                                                        dateParts.setDD(dayNumber);
                                                    }
                                                }
                                            }
                                            else
                                            {
                                                if (strDateParts[1].length() == 4)
                                                {
                                                    dateParts = new DateParts();
                                                    dateParts.setYYYY(Integer.parseInt(strDateParts[1]));
                                                    int monthNumber = getMonthNumber(strDateParts[0]);
                                                    if (monthNumber != -1)
                                                    {
                                                        dateParts.setMM(monthNumber);
                                                    }
                                                }
                                                else
                                                {
                                                    LOGGER.error("Not Matched #2: " + dateStr);
                                                }
                                            }
                                        }
                                    }
                                }
                                else
                                {
                                    if (patTomorrow.matcher(dateStr).find())
                                    {
                                        dateParts = new DateParts();

                                        if ((baseDay == 31 && (baseMonth == 1 || baseMonth == 3 || baseMonth == 5 || baseMonth == 7 || baseMonth == 8 || baseMonth == 10 || baseMonth == 12)) || (baseDay == 30 && (baseMonth == 4 || baseMonth == 6 || baseMonth == 11)) || (baseDay == 29 && baseMonth == 2 && isLeapYear(baseYear)) || (baseDay == 28 && baseMonth == 2 && !isLeapYear(baseYear)))
                                        {
                                            dateParts.setDD(1);
                                            if (baseMonth == 12)
                                            {
                                                dateParts.setMM(1);
                                                dateParts.setYYYY(baseYear + 1);
                                            }
                                            else
                                            {
                                                dateParts.setMM(baseMonth + 1);
                                                dateParts.setYYYY(baseYear);
                                            }
                                        }
                                        else
                                        {
                                            dateParts.setDD(baseDay + 1);
                                            dateParts.setMM(baseMonth);
                                            dateParts.setYYYY(baseYear);
                                        }
                                    }
                                    else
                                    {
                                        if (patNextNYears.matcher(dateStr).find())
                                        {
                                            String[] strDateParts = dateStr.split("\\s+");
                                            if (strDateParts.length == 3)
                                            {
                                                if (strDateParts[2].equals("year") || strDateParts[2].equals("years") || strDateParts[2].equals("quarter") || strDateParts[2].equals("quarters") || strDateParts[2].equals("month") || strDateParts[2].equals("months") || strDateParts[2].equals("week") || strDateParts[2].equals("weeks"))
                                                {
                                                    int numUnits = -1;
                                                    if (isNumeric(strDateParts[1]))
                                                    {
                                                        numUnits = Integer.parseInt(strDateParts[1]);
                                                    }
                                                    else
                                                    {
                                                        numUnits = convertSingleDigitNumberInWordsToNumber(strDateParts[1]);
                                                    }
                                                    if (numUnits != -1)
                                                    {

                                                        GregorianCalendar gregcal = new GregorianCalendar();
                                                        gregcal.set(GregorianCalendar.DATE, baseDay);
                                                        gregcal.set(GregorianCalendar.MONTH, (baseMonth - 1)); // zero based months
                                                        gregcal.set(GregorianCalendar.YEAR, baseYear);

                                                        if (strDateParts[2].startsWith("year"))
                                                        {
                                                            gregcal.add(GregorianCalendar.YEAR, numUnits);
                                                        }
                                                        else
                                                        {
                                                            if (strDateParts[2].startsWith("quarter"))
                                                            {
                                                                gregcal.add(GregorianCalendar.MONTH, (3 * numUnits));
                                                            }
                                                            else
                                                            {
                                                                if (strDateParts[2].startsWith("month"))
                                                                {
                                                                    gregcal.add(GregorianCalendar.MONTH, numUnits);
                                                                }
                                                                else
                                                                {
                                                                    if (strDateParts[2].startsWith("weeks"))
                                                                    {
                                                                        gregcal.add(GregorianCalendar.DATE, (7 * numUnits));
                                                                    }
                                                                }
                                                            }
                                                        }

                                                        dateParts = new DateParts();
                                                        dateParts.setDD(gregcal.get(GregorianCalendar.DATE));
                                                        dateParts.setMM((gregcal.get(GregorianCalendar.MONTH) + 1)); // zero based months
                                                        dateParts.setYYYY(gregcal.get(GregorianCalendar.YEAR));
                                                    }
                                                }
                                            }
                                        }
                                        else
                                        {
                                            if (patNYearsFromNow.matcher(dateStr).find())
                                            {
                                                String[] strDateParts = dateStr.split("\\s+");
                                                if (strDateParts.length == 4 || strDateParts.length == 3)
                                                {
                                                    if (strDateParts[1].equals("year") || strDateParts[1].equals("years") || strDateParts[1].equals("quarter") || strDateParts[1].equals("quarters") || strDateParts[1].equals("month") || strDateParts[1].equals("months") || strDateParts[1].equals("week") || strDateParts[1].equals("weeks"))
                                                    {
                                                        int numUnits = -1;
                                                        if (isNumeric(strDateParts[0]))
                                                        {
                                                            numUnits = Integer.parseInt(strDateParts[0]);
                                                        }
                                                        else
                                                        {
                                                            numUnits = convertSingleDigitNumberInWordsToNumber(strDateParts[0]);
                                                        }
                                                        if (numUnits != -1)
                                                        {

                                                            GregorianCalendar gregcal = new GregorianCalendar();
                                                            gregcal.set(GregorianCalendar.DATE, baseDay);
                                                            gregcal.set(GregorianCalendar.MONTH, (baseMonth - 1)); // zero based months
                                                            gregcal.set(GregorianCalendar.YEAR, baseYear);

                                                            if (strDateParts[1].startsWith("year"))
                                                            {
                                                                gregcal.add(GregorianCalendar.YEAR, numUnits);
                                                            }
                                                            else
                                                            {
                                                                if (strDateParts[1].startsWith("quarter"))
                                                                {
                                                                    gregcal.add(GregorianCalendar.MONTH, (3 * numUnits));
                                                                }
                                                                else
                                                                {
                                                                    if (strDateParts[1].startsWith("month"))
                                                                    {
                                                                        gregcal.add(GregorianCalendar.MONTH, numUnits);
                                                                    }
                                                                    else
                                                                    {
                                                                        if (strDateParts[1].startsWith("weeks"))
                                                                        {
                                                                            gregcal.add(GregorianCalendar.DATE, (7 * numUnits));
                                                                        }
                                                                    }
                                                                }
                                                            }

                                                            dateParts = new DateParts();
                                                            dateParts.setDD(gregcal.get(GregorianCalendar.DATE));
                                                            dateParts.setMM((gregcal.get(GregorianCalendar.MONTH) + 1)); // zero based months
                                                            dateParts.setYYYY(gregcal.get(GregorianCalendar.YEAR));
                                                        }
                                                    }
                                                }
                                            }
                                            else
                                            {
                                                //if (dateStr.matches(strMonthOnly))
                                                if (patMonthOnly.matcher(dateStr).find())
                                                {
                                                    int monthNumber = getMonthNumber(dateStr);
                                                    if (monthNumber != -1)
                                                    {
                                                        dateParts = new DateParts();
                                                        dateParts.setMM(monthNumber);
                                                        dateParts.setYYYY(baseYear);
                                                    }
                                                }
                                                else
                                                {
                                                    //if (dateStr.matches(strMonthAndDay))
                                                    if (patMonthAndDay.matcher(dateStr).find())
                                                    {
                                                        String[] strDateParts = dateStr.split("\\s+");
                                                        if (strDateParts.length == 2)
                                                        {
                                                            int monthNumber = getMonthNumber(strDateParts[0]);
                                                            if (monthNumber != -1)
                                                            {
                                                                int dayNumber = getDayNumber(strDateParts[1]);
                                                                if (dayNumber != -1)
                                                                {
                                                                    dateParts = new DateParts();
                                                                    dateParts.setDD(dayNumber);
                                                                    dateParts.setMM(monthNumber);
                                                                    dateParts.setYYYY(baseYear);
                                                                }
                                                            }
                                                        }
                                                    }
                                                    else
                                                    {
                                                        LOGGER.error("Not Matched #3: " + dateStr);
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        catch (Exception ex)
        {
            LOGGER.error(ex.getMessage(), ex);
        }

        return dateParts;
    }

    private int getMonthNumber(String monthName)
    {
        int monthNumber = -1;
        String monthNameLower = monthName.toLowerCase();
        if (monthNameLower.equals("jan") || monthNameLower.equals("jan.") || monthNameLower.equals("january") || monthNameLower.equals("january,"))
        {
            monthNumber = 1;
        }
        else
        {
            if (monthNameLower.equals("feb") || monthNameLower.equals("feb.") || monthNameLower.equals("february") || monthNameLower.equals("february,"))
            {
                monthNumber = 2;
            }
            else
            {
                if (monthNameLower.equals("mar") || monthNameLower.equals("mar.") || monthNameLower.equals("march") || monthNameLower.equals("march,"))
                {
                    monthNumber = 3;
                }
                else
                {
                    if (monthNameLower.equals("apr") || monthNameLower.equals("apr.") || monthNameLower.equals("april") || monthNameLower.equals("april,"))
                    {
                        monthNumber = 4;
                    }
                    else
                    {
                        if (monthNameLower.equals("may") || monthNameLower.equals("may,"))
                        {
                            monthNumber = 5;
                        }
                        else
                        {
                            if (monthNameLower.equals("jun") || monthNameLower.equals("jun.") || monthNameLower.equals("june") || monthNameLower.equals("june,"))
                            {
                                monthNumber = 6;
                            }
                            else
                            {
                                if (monthNameLower.equals("jul") || monthNameLower.equals("jul.") || monthNameLower.equals("july") || monthNameLower.equals("july,"))
                                {
                                    monthNumber = 7;
                                }
                                else
                                {
                                    if (monthNameLower.equals("aug") || monthNameLower.equals("aug.") || monthNameLower.equals("august") || monthNameLower.equals("august,"))
                                    {
                                        monthNumber = 8;
                                    }
                                    else
                                    {
                                        if (monthNameLower.equals("sep") || monthNameLower.equals("sep.") || monthNameLower.equals("september") || monthNameLower.equals("september,"))
                                        {
                                            monthNumber = 9;
                                        }
                                        else
                                        {
                                            if (monthNameLower.equals("oct") || monthNameLower.equals("oct.") || monthNameLower.equals("october") || monthNameLower.equals("october,"))
                                            {
                                                monthNumber = 10;
                                            }
                                            else
                                            {
                                                if (monthNameLower.equals("nov") || monthNameLower.equals("nov.") || monthNameLower.equals("november") || monthNameLower.equals("november,"))
                                                {
                                                    monthNumber = 11;
                                                }
                                                else
                                                {
                                                    if (monthNameLower.equals("dec") || monthNameLower.equals("dec.") || monthNameLower.equals("december") || monthNameLower.equals("december,"))
                                                    {
                                                        monthNumber = 12;
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        return monthNumber;
    }

    private int getDayNumber(String strDay)
    {
        int dayNumber = -1;
        String strDayNum = strDay;
        char[] unwantedChar = new char[]
        {
            ','
        };
        CommonUtil genutil = new CommonUtil();
        strDayNum = genutil.trimCharacters(unwantedChar, strDayNum);
        if (strDayNum.endsWith("st") || strDayNum.endsWith("nd") || strDayNum.endsWith("rd") || strDayNum.endsWith("th"))
        {
            strDayNum = strDayNum.substring(0, (strDayNum.length() - 2)).trim();
        }
        else
        {
            strDayNum = strDayNum.trim();
        }

        try
        {
            dayNumber = Integer.parseInt(strDayNum);
        }
        catch (Exception ex)
        {
            LOGGER.error(ex.getMessage() + " - " + strDay, ex);
        }
        return dayNumber;
    }

    private int convertSingleDigitNumberInWordsToNumber(String numberInWords)
    {
        int num = -1;
        numberInWords = numberInWords.toLowerCase().trim();
        if (numberInWords.equals("one"))
        {
            num = 1;
        }
        else
        {
            if (numberInWords.equals("two"))
            {
                num = 2;
            }
            else
            {
                if (numberInWords.equals("three"))
                {
                    num = 3;
                }
                else
                {
                    if (numberInWords.equals("four"))
                    {
                        num = 4;
                    }
                    else
                    {
                        if (numberInWords.equals("five"))
                        {
                            num = 5;
                        }
                        else
                        {
                            if (numberInWords.equals("six"))
                            {
                                num = 6;
                            }
                            else
                            {
                                if (numberInWords.equals("seven"))
                                {
                                    num = 7;
                                }
                                else
                                {
                                    if (numberInWords.equals("eight"))
                                    {
                                        num = 8;
                                    }
                                    else
                                    {
                                        if (numberInWords.equals("nine"))
                                        {
                                            num = 9;
                                        }
                                        else
                                        {
                                            if (numberInWords.equals("ten"))
                                            {
                                                num = 10;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        return num;
    }

    private boolean isLeapYear(int year)
    {
        boolean leapYear = false;
        if (year % 4 == 0)
        {
            if (year % 400 == 0)
            {
                leapYear = true;
            }
            else
            {
                if (year % 100 != 0)
                {
                    leapYear = true;
                }
            }
        }
        return leapYear;
    }

    /**
     * Checks for whole numbers only, decimals are not allowed.
     */
    private boolean isNumeric(String str)
    {
        boolean numeric = true;
        for (int i = 0; i < str.length(); ++i)
        {
            int asciiCode = (int) str.charAt(i);
            if (asciiCode < 48 || asciiCode > 57)
            {
                numeric = false;
                break;
            }
        }
        return numeric;
    }

}
