/*
 * This Java source file is completely Free and Open-Source, 
 * released under the MIT License.
 *
 * Copyright (2012) Ashish Nawal Saharia (ashish.saharia@yahoo.co.in)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to deal 
 * in the Software without restriction, including without limitation the rights 
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
 * OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 */

/*
 * DateUtil.java
 *
 * Created on November 11, 2008, 7:46 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package net.infantai.util.common;

import java.sql.Timestamp;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Locale;
import java.util.TimeZone;


public class DateUtil
{
    
    public DateUtil()
    {
        // Default constructor, nothing to do here.
    }
    
    public static String getDateAsString(GregorianCalendar gregcal, boolean dateWithoutTime, boolean format24Hour)
    {
        String dateString;
        int hour = gregcal.get(GregorianCalendar.HOUR_OF_DAY);
        if (!format24Hour)
        {
            if (hour > 12)
            {
                hour = hour - 12;
            }
        }
        dateString = gregcal.getDisplayName(GregorianCalendar.DAY_OF_WEEK, Calendar.SHORT_FORMAT, Locale.ENGLISH);
        dateString += " " + zeroPad(gregcal.get(GregorianCalendar.DAY_OF_MONTH));
        dateString += " " + gregcal.getDisplayName(GregorianCalendar.MONTH, Calendar.SHORT_FORMAT, Locale.ENGLISH);
        dateString += " " + gregcal.get(GregorianCalendar.YEAR);
        if (!dateWithoutTime)
        {
            dateString += " " + zeroPad(hour) + ":" + zeroPad(gregcal.get(GregorianCalendar.MINUTE)) + ":" + zeroPad(gregcal.get(GregorianCalendar.SECOND));
            if (!format24Hour)
            {
                dateString += " " + (gregcal.get(GregorianCalendar.AM_PM) == 0 ? "AM" : "PM");
            }        
        }
        return dateString;
    }
    
    public static String getDateAsStringLongFormat(GregorianCalendar gregcal, boolean dateWithoutTime, boolean format24Hour, boolean includeMilliseconds, boolean includeTimeZoneOffset)
    {
        String dateString;
        int hour = gregcal.get(GregorianCalendar.HOUR_OF_DAY);
        if (!format24Hour)
        {
            if (hour > 12)
            {
                hour = hour - 12;
            }
        }
        dateString = gregcal.getDisplayName(GregorianCalendar.DAY_OF_WEEK, Calendar.LONG_FORMAT, Locale.ENGLISH);
        dateString += ", " + gregcal.getDisplayName(GregorianCalendar.MONTH, Calendar.LONG_FORMAT, Locale.ENGLISH);
        dateString += " " + zeroPad(gregcal.get(GregorianCalendar.DAY_OF_MONTH));
        dateString += ", " + gregcal.get(GregorianCalendar.YEAR);
        if (!dateWithoutTime)
        {
            dateString += ", " + zeroPad(hour) + ":" + zeroPad(gregcal.get(GregorianCalendar.MINUTE)) + ":" + zeroPad(gregcal.get(GregorianCalendar.SECOND));
            
            if (includeMilliseconds)
            {
                dateString += "." + zeroPad(gregcal.get(GregorianCalendar.MILLISECOND), 3);
            }
            
            if (!format24Hour)
            {
                dateString += " " + (gregcal.get(GregorianCalendar.AM_PM) == 0 ? "AM" : "PM");
            }            
            
            if (includeTimeZoneOffset)
            {
                TimeZone defaultTZ = TimeZone.getDefault();
                int tzOffsetMS = defaultTZ.getRawOffset();
                String tzOffset = String.format("%s%02d:%02d", tzOffsetMS < 0 ? "-" : "+", Math.abs(tzOffsetMS) / 3600000, (Math.abs(tzOffsetMS) / 60000) % 60);

                dateString += " UTC"+tzOffset;
            }
        }
        return dateString;
    }    
    
    
    /**
     * Returns a textual representation of the time value specified by datetimemillisec. 
     * <br><br>If format24Hour = true, the format of the returned string is <b>EEE DD MMM yyyy HH:mm:ss</b>. <br>Example: Wed 02 Jan 2019 14:52:43
     * <br><br>If format24Hour = false, the format of the returned string is <b>EEE DD MMM yyyy HH:mm:ss a</b>. <br>Example: Wed 02 Jan 2019 03:06:17 PM
     * @param datetimemillisec
     * @param dateWithoutTime
     * @param format24Hour
     * @return A String representation of the time value specified by datetimemillisec. Example: <i>Wed 02 Jan 2019 14:52:43</i>
     */
    public static String getDateAsString(long datetimemillisec, boolean dateWithoutTime, boolean format24Hour)
    {
        GregorianCalendar gregcal = new GregorianCalendar();
        gregcal.setTimeInMillis(datetimemillisec);
        return getDateAsString(gregcal, dateWithoutTime, format24Hour);
    }
    
    /**
     * Returns a textual representation of the time value specified by datetimemillisec. 
     * <br><br>If format24Hour = true, the format of the returned string is <b>EEE DD MMM yyyy HH:mm:ss</b>. <br>Example: Wed 02 Jan 2019 14:52:43
     * <br><br>If format24Hour = false, the format of the returned string is <b>EEE DD MMM yyyy HH:mm:ss a</b>. <br>Example: Wed 02 Jan 2019 03:06:17 PM
     * @param datetimemillisec
     * @param dateWithoutTime
     * @param format24Hour
     * @param includeMilliseconds
     * @param includeTimeZoneOffset
     * @return A String representation of the time value specified by datetimemillisec. Example: <i>Wed 02 Jan 2019 14:52:43.081 UTC+05:30</i>
     */
    public static String getDateAsStringLongFormat(long datetimemillisec, boolean dateWithoutTime, boolean format24Hour, boolean includeMilliseconds, boolean includeTimeZoneOffset)
    {
        GregorianCalendar gregcal = new GregorianCalendar();
        gregcal.setTimeInMillis(datetimemillisec);
        return getDateAsStringLongFormat(gregcal, dateWithoutTime, format24Hour, includeMilliseconds, includeTimeZoneOffset);
    }    

    /**
     * Returns a datetime string representing the time value specified by datetimemillisec. 
     * The format of the returned datetime string is <b>yyyy-MM-dd HH:mm:ss.sss</b>. <br>Example: 2019-01-02 20:31:34.601
     * @param datetimemillisec
     * @param includeMilliseconds
     * @return A String representation of the time value specified by datetimemillisec. Example: <i>2019-01-02 14:52:43.081</i>
     */
    public static String getDateStringForDB(long datetimemillisec, boolean includeMilliseconds)
    {
        GregorianCalendar gregcal = new GregorianCalendar();
        gregcal.setTimeInMillis(datetimemillisec);
        String dateString = gregcal.get(GregorianCalendar.YEAR) + "-" + zeroPad(gregcal.get(GregorianCalendar.MONTH) + 1) + "-" + zeroPad(gregcal.get(GregorianCalendar.DAY_OF_MONTH));
        dateString += " " + zeroPad(gregcal.get(GregorianCalendar.HOUR_OF_DAY)) + ":" + zeroPad(gregcal.get(GregorianCalendar.MINUTE)) + ":" + zeroPad(gregcal.get(GregorianCalendar.SECOND));
        if (includeMilliseconds)
        {
            dateString += "." + zeroPad(gregcal.get(GregorianCalendar.MILLISECOND), 3);
        }
        return dateString;
    }
    
    /**
     * Returns a datetime string representing the time value specified by datetimemillisec. 
     * <br><br>If includeTimeZoneOffset = true and includeTimeZoneIdString = true, the format of the returned datetime string is <b>yyyy-MM-ddTHH:mm:ss.sss {TimeZone Offset from UTC} {TimeZoneIdString}</b>. There's a 'T' between the Date and Time parts of the returned string. <br>Example: 2019-01-02T14:25:23.542 +05:30 [Asia/Calcutta]
     * <br><br>If includeTimeZoneOffset = true and includeTimeZoneIdString = false, the format of the returned datetime string is <b>yyyy-MM-ddTHH:mm:ss.sss {TimeZone Offset from UTC}</b>. There's a 'T' between the Date and Time parts of the returned string. <br>Example: 2019-01-02T14:26:13.195 +05:30
     * <br><br>If includeTimeZoneOffset = false and includeTimeZoneIdString = true, the format of the returned datetime string is <b>yyyy-MM-dd HH:mm:ss.sss</b>. Parameter includeTimeZoneIdString is ignored if includeTimeZoneOffset = false. There's a Space between the Date and Time parts of the returned string. <br>Example: 2019-01-02 14:37:35.347
     * <br><br>If includeTimeZoneOffset = false and includeTimeZoneIdString = false, the format of the returned datetime string is <b>yyyy-MM-dd HH:mm:ss.sss</b>. There's a Space between the Date and Time parts of the returned string. <br>Example: 2019-01-02 14:37:35.347
     * @param datetimemillisec
     * @param includeTimeZoneOffset
     * @param includeTimeZoneIdString
     * @return A String representation of the time value specified by datetimemillisec.
     */
    public static String getDateStringWithMilliseconds(long datetimemillisec, boolean includeTimeZoneOffset, boolean includeTimeZoneIdString)
    {
        GregorianCalendar gregcal = new GregorianCalendar();
        gregcal.setTimeInMillis(datetimemillisec);
        StringBuilder sbDateString = new StringBuilder();
        sbDateString.append(gregcal.get(GregorianCalendar.YEAR));
        sbDateString.append('-');
        sbDateString.append(zeroPad(gregcal.get(GregorianCalendar.MONTH) + 1, 2));
        sbDateString.append('-');
        sbDateString.append(zeroPad(gregcal.get(GregorianCalendar.DAY_OF_MONTH), 2));
        if (includeTimeZoneOffset)
        {
            sbDateString.append('T');
        }
        else
        {
            sbDateString.append(' ');
        }
        sbDateString.append(zeroPad(gregcal.get(GregorianCalendar.HOUR_OF_DAY), 2));
        sbDateString.append(':');
        sbDateString.append(zeroPad(gregcal.get(GregorianCalendar.MINUTE), 2));
        sbDateString.append(':');
        sbDateString.append(zeroPad(gregcal.get(GregorianCalendar.SECOND), 2));
        sbDateString.append('.');
        sbDateString.append(zeroPad(gregcal.get(GregorianCalendar.MILLISECOND), 3));
        if (includeTimeZoneOffset)
        {
            TimeZone defaultTZ = TimeZone.getDefault();
            int tzOffsetMS = defaultTZ.getRawOffset();
            String tzOffset = String.format("%s%02d:%02d", tzOffsetMS < 0 ? "-" : "+", Math.abs(tzOffsetMS) / 3600000, (Math.abs(tzOffsetMS) / 60000) % 60);
            sbDateString.append(' ');
            sbDateString.append(tzOffset);
            if (includeTimeZoneIdString)
            {
                sbDateString.append(' ');
                sbDateString.append('[');
                sbDateString.append(defaultTZ.getID());
                sbDateString.append(']');            
            }
        }
        return sbDateString.toString();
    }  
    
    /**
     * Returns the current UTC timestamp as a String, based on the system's DateTime and TimeZone settings.
     * The format of the returned datetime string is <b>yyyy-MM-dd HH:mm:ss.sss</b>. <br>Example: 2019-01-02T14:25:23.542 +05:30 [Asia/Calcutta]
     * @return A String representation of the current UTC time, as determined by the current time and time-zone set on the system. 
     */
    public static Timestamp getUTCTimestamp() 
    {
       Calendar cal = Calendar.getInstance();
       return new Timestamp(cal.getTimeInMillis() - cal.get(Calendar.ZONE_OFFSET) - cal.get(Calendar.DST_OFFSET));
    }    
    
    public static String zeroPad(int digit)
    {
        String paddedNum;
        if (digit < 10)
        {
            paddedNum = "0" + digit;
        }
        else
        {
            paddedNum = Integer.toString(digit);
        }
        return paddedNum;
    }
    
    public static String zeroPad(int number, int finalPaddedLength)
    {
        StringBuilder paddedNum = new StringBuilder();
        String strNumber = Integer.toString(number);
        int zerosToPad = finalPaddedLength - strNumber.length();
        for (int i = 0; i < zerosToPad; ++i)
        {
            paddedNum.append('0');
        }
        paddedNum.append(strNumber);
        return paddedNum.toString();
    }
    
    public static void main(String[] args)
    {
        System.out.println(getDateStringWithMilliseconds(System.currentTimeMillis(), true, true));
        System.out.println(getDateStringWithMilliseconds(System.currentTimeMillis(), true, false));
        System.out.println(getDateStringWithMilliseconds(System.currentTimeMillis(), false, true));
        System.out.println(getDateStringWithMilliseconds(System.currentTimeMillis(), false, false));
        System.out.println("UTC: " + getUTCTimestamp());
        System.out.println(getDateAsString(System.currentTimeMillis(), false, false));
        System.out.println(getDateStringForDB(System.currentTimeMillis(), false));
        System.out.println(getDateStringForDB(System.currentTimeMillis(), true));   
        GregorianCalendar gregcal = new GregorianCalendar();
        System.out.println(getDateAsString(gregcal, false, true));   
        System.out.println(getDateAsStringLongFormat(gregcal, false, true, true, true));   
        System.out.println(getDateAsStringLongFormat(gregcal, false, true, false, false));   
        System.out.println(getDateAsStringLongFormat(gregcal, true, true, true, true));   
    }
    
}
