/*
 * This Java source file is completely Free and Open-Source, 
 * released under the MIT License.
 *
 * Copyright (2012) Ashish Nawal Saharia (ashish.saharia@yahoo.co.in)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy 
 * of this software and associated documentation files (the "Software"), to deal 
 * in the Software without restriction, including without limitation the rights 
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
 * copies of the Software, and to permit persons to whom the Software is 
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, 
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR 
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
 * OR OTHER DEALINGS IN THE SOFTWARE.
 * 
 */

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package net.infantai.util.string.similarity;

import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

/**
 *
 * @author Ashish (ashish.saharia@yahoo.co.in)
 */
public class SimilarityCheckerArthaat implements SimilarityChecker
{
    
    private Map<String, Set<String>> synonyms = new LinkedHashMap<String, Set<String>>();
    private Set<String> tokensToDiscard = new LinkedHashSet<String>();
    
    public SimilarityCheckerArthaat()
    {
    
    }
    
    public SimilarityCheckerArthaat(Map<String, Set<String>> synonyms, Set<String> tokensToDiscard)
    {
        this.synonyms = synonyms;
        this.tokensToDiscard = tokensToDiscard;
    }
    
    
    public static void main(String[] args)
    {
        String str1 = "abdomen pain";
        String str2 = "abdominal pain";
        SimilarityCheckerArthaat scc = new SimilarityCheckerArthaat();
        System.out.println(scc.getStringSimilarity(str1, str2));
//        SimilarityCheckerCosine sccOld = new SimilarityCheckerCosine();
//        System.out.println(sccOld.getStringSimilarity(str1, str2));
    }
    
    private Set<Character> getCharacters(String input)
    {
        Set<Character> characters = new HashSet<Character>();
        for (int i = 0; i < input.length(); ++i)
        {
            characters.add(new Character(input.charAt(i)));
        }
        return characters;
    }

    @Override
    public double getStringSimilarity(String str1, String str2)
    {
        double cosineSimilarity = 0.0;
        Set<Character> str1Tokens = getCharacters(str1);
        Set<Character> str2Tokens = getCharacters(str2);
        Set<Character> allCh = new HashSet<Character>(str1Tokens.size() + str2Tokens.size());
        allCh.addAll(str1Tokens);
        allCh.addAll(str2Tokens);
        int commonTermCount = (str1Tokens.size() + str2Tokens.size()) - allCh.size();
        cosineSimilarity = (double) commonTermCount / (Math.pow(str1Tokens.size(), 0.5f) * Math.pow(str2Tokens.size(), 0.5f));
        return Math.min(1.0f, cosineSimilarity);
    }    
    
}
